"use strict";
/* ============================================================================
 * Copyright (c) Palo Alto Networks
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 * ========================================================================== */
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTagPageMD = exports.createInfoPageMD = exports.createApiPageMD = void 0;
const createAuthentication_1 = require("./createAuthentication");
const createContactInfo_1 = require("./createContactInfo");
// import { createDeprecationNotice } from "./createDeprecationNotice";
const createDescription_1 = require("./createDescription");
const createDownload_1 = require("./createDownload");
const createEndpoint_1 = require("./createEndpoint");
const createLicense_1 = require("./createLicense");
const createLogo_1 = require("./createLogo");
const createParamsDetails_1 = require("./createParamsDetails");
// import { createRequestBodyDetails } from "./createRequestBodyDetails";
const createRequestSchema_1 = require("./createRequestSchema");
const createStatusCodes_1 = require("./createStatusCodes");
const createTermsOfService_1 = require("./createTermsOfService");
// import { createVendorExtensions } from "./createVendorExtensions";
const createVersionBadge_1 = require("./createVersionBadge");
const utils_1 = require("./utils");
// interface Props {
//   title: string;
//   body: {
//     content?: {
//       [key: string]: MediaTypeObject;
//     };
//     description?: string;
//     required?: boolean;
//   };
// }
function createApiPageMD({ 
// title,
api: { 
// deprecated,
// "x-deprecated-description": deprecatedDescription,
description, 
// extensions,
parameters, requestBody, responses, path, method, }, }) {
    // return render([
    //   `import ApiTabs from "@theme/ApiTabs";\n`,
    //   `import MimeTabs from "@theme/MimeTabs";\n`,
    //   `import ParamsItem from "@theme/ParamsItem";\n`,
    //   `import ResponseSamples from "@theme/ResponseSamples";\n`,
    //   `import SchemaItem from "@theme/SchemaItem";\n`,
    //   `import SchemaTabs from "@theme/SchemaTabs";\n`,
    //   `import DiscriminatorTabs from "@theme/DiscriminatorTabs";\n`,
    //   `import SchemaTable from "@theme/SchemaTable";\n`,
    //   `import RequestBodyDetails from "@theme/RequestBodyDetails";\n`,
    //   `import TabItem from "@theme/TabItem";\n\n`,
    //   `## ${title.replace(lessThan, "&lt;").replace(greaterThan, "&gt;")}\n\n`,
    //   frontMatter.show_extensions && createVendorExtensions(extensions),
    //   createDeprecationNotice({ deprecated, description: deprecatedDescription }),
    //   createDescription(description),
    //   createParamsDetails({ parameters, type: "path" }),
    //   createParamsDetails({ parameters, type: "query" }),
    //   createParamsDetails({ parameters, type: "header" }),
    //   createParamsDetails({ parameters, type: "cookie" }),
    //   createRequestBodyDetails({
    //     title: "Request Body",
    //     body: requestBody,
    //   } as Props),
    //   createStatusCodes({ responses }),
    // ]);
    return (0, utils_1.render)([
        `import Layout from "@theme/Layout";\n`,
        `import Endpoint from "@theme/Endpoint";\n`,
        `import ParamDetails from "@theme/ParamDetails";\n`,
        `import RequestBodyDetailsBase from "@theme/RequestBodyDetails/base";\n`,
        `import RequestBodyDetails from "@theme/RequestBodyDetails";\n`,
        `import Table from "@theme/SchemaTable";\n`,
        `import MimeTabs from "@theme/MimeTabs";\n`,
        `import ApiTabs from "@theme/ApiTabs";\n`,
        `import SchemaTabs from "@theme/SchemaTabs";\n`,
        `import ResponseSamples from "@theme/ResponseSamples";\n`,
        // create("Layout", {
        //   children: [
        description ? `\n\n${description.trim()}\n\n` : "",
        (0, createEndpoint_1.createEndpoint)(method, path),
        (0, createParamsDetails_1.createParamsDetails)({ parameters, type: "path" }),
        (0, createParamsDetails_1.createParamsDetails)({ parameters, type: "query" }),
        (0, createParamsDetails_1.createParamsDetails)({ parameters, type: "header" }),
        (0, createParamsDetails_1.createParamsDetails)({ parameters, type: "cookie" }),
        requestBody
            ? (0, createRequestSchema_1.createRequestSchema)({ body: requestBody, title: "Request body" })
            : "",
        (0, createStatusCodes_1.createStatusCodes)({ responses }),
        //   ],
        // }
        // ),
    ]);
}
exports.createApiPageMD = createApiPageMD;
function createInfoPageMD({ info: { title, version, description, contact, license, termsOfService, logo, darkLogo, }, securitySchemes, downloadUrl, }) {
    return (0, utils_1.render)([
        `import ApiLogo from "@theme/ApiLogo";\n`,
        `import SchemaTabs from "@theme/SchemaTabs";\n`,
        `import TabItem from "@theme/TabItem";\n`,
        `import Export from "@theme/ApiDemoPanel/Export";\n\n`,
        (0, createVersionBadge_1.createVersionBadge)(version),
        (0, createDownload_1.createDownload)(downloadUrl),
        `# ${title.replace(utils_1.lessThan, "&lt;").replace(utils_1.greaterThan, "&gt;")}\n\n`,
        (0, createLogo_1.createLogo)(logo, darkLogo),
        (0, createDescription_1.createDescription)(description),
        (0, createAuthentication_1.createAuthentication)(securitySchemes),
        (0, createContactInfo_1.createContactInfo)(contact),
        (0, createTermsOfService_1.createTermsOfService)(termsOfService),
        (0, createLicense_1.createLicense)(license),
    ]);
}
exports.createInfoPageMD = createInfoPageMD;
function createTagPageMD({ tag: { description } }) {
    return (0, utils_1.render)([(0, createDescription_1.createDescription)(description)]);
}
exports.createTagPageMD = createTagPageMD;
